from time import sleep

from lannerpsp import RFM

rfm = RFM()

print(rfm.get_sim_status())  # It should print 0.

# Power off the mini-PCIe LTE module.
# 01: mPCIe -> off, M.2 -> on
rfm.set_power_status(1)

# Switch to SIM4 slot.
# 10: mPCIe -> second sim (SIM4), M.2 -> first sim (SIM1)
rfm.set_sim_status(2)

# Power on the mini-PCIe LTE module.
# 11: mPCIe -> on,  M.2 -> on
rfm.set_power_status(3)

# It will take about 15 seconds for the SIM card to be switched successfully.
sleep(15)



from os import geteuid
from time import sleep


def is_root() -> bool:
    """Check if is root user."""
    return geteuid() == 0


def show_delay(secs: int) -> None:
    """Flush delay time (seconds) to stdout."""
    index = int(secs)
    while index > 0:
        print(f"{index}. ", end="", flush=True)
        index -= 1
        sleep(1.0)
    print(f"{index}.")



"""
Platform: LEC-7230
PSP version: 3.0.0
OS version:
- Debian 10: All pass
SDK:
- sdk_dll: Done.
- sdk_gpio: Not implement yet, don't know how to use this SDK.
- sdk_hwm: Done, except `-testop [seconds]`, don't know how to use this function.
- sdk_odm: Done.
- sdk_swr: Not implement yet, don't know how to set the button from hardware reset to software reset.
- sdk_wdt: Done.
"""
from time import sleep

import pytest

from lannerpsp import *

DELAY_TIME = 2  # Seconds.


class TestPSP:

    def test_init(self):
        with PSP():
            pass


class TestDLL:
    dll = DLL()

    def test_get_version(self):
        version = self.dll.get_version()
        assert version.dll_major == 3
        assert version.dll_minor == 0
        assert version.dll_build == 0
        assert version.platform_id == "LEC-7230"
        assert version.board_major == 1
        assert version.board_minor == 1
        assert version.board_build == 0

    def test_get_bios_id(self):
        assert self.dll.get_bios_id() == 'LEB-7230L Ver.MI0 04/10/2018'


class TestHWM:
    hwm = HWM()

    def test_list_supported_sensors(self):
        supported_sensors = self.hwm.list_supported_sensors()
        assert sorted([s.sid for s in supported_sensors]) == sorted([4, 5, 41, 42, 45, 46, 47, 49, 50])
        assert supported_sensors[0].sid == 4
        assert supported_sensors[0].name == "HWMID_TEMP_SYS1"
        assert supported_sensors[1].sid == 5
        assert supported_sensors[1].name == "HWMID_TEMP_SYS2"
        assert supported_sensors[2].sid == 41
        assert supported_sensors[2].name == "HWMID_VCORE_CPU1"
        assert supported_sensors[3].sid == 42
        assert supported_sensors[3].name == "HWMID_VCORE_CPU2"
        assert supported_sensors[4].sid == 45
        assert supported_sensors[4].name == "HWMID_VOLT_P12V"
        assert supported_sensors[5].sid == 46
        assert supported_sensors[5].name == "HWMID_VOLT_P5V"
        assert supported_sensors[6].sid == 47
        assert supported_sensors[6].name == "HWMID_VOLT_P3V3"
        assert supported_sensors[7].sid == 49
        assert supported_sensors[7].name == "HWMID_VOLT_P3V3SB"
        assert supported_sensors[8].sid == 50
        assert supported_sensors[8].name == "HWMID_VOLT_VBAT"


class TestComPort:

    def test_set_all(self):
        com1 = COMPort(1)
        com1.set_mode(232)
        com1.set_termination(False)
        com1_info = com1.get_info()
        assert com1_info.num == 1
        assert com1_info.mode == 232
        assert com1_info.mode_str == "RS-232"
        assert com1_info.termination is False
        assert com1_info.termination_str == "Disabled"
        com2 = COMPort(2)
        com2.set_mode(485)
        com2.set_termination(True)
        com2_info = com2.get_info()
        assert com2_info.num == 2
        assert com2_info.mode == 485
        assert com2_info.mode_str == "RS-485"
        assert com2_info.termination is True
        assert com2_info.termination_str == "Enabled"

    def test_init_out_of_range(self):
        with pytest.raises(PSPInvalid) as e:
            COMPort(3)
        assert str(e.value) == "'num' can only be set to (1~2) on this platform"

    def test_set_mode_out_of_range(self):
        com1 = COMPort(1)
        with pytest.raises(ValueError):
            com1.set_mode(666)


class TestWDT:
    wdt = WDT()

    def test_get_info(self):
        wdt_info = self.wdt.get_info()
        assert wdt_info.type == "SuperIO"
        assert wdt_info.max_count == 255
        assert wdt_info.is_minute_support is True

    def test_enable(self):
        self.wdt.enable(10, 2)

    def test_reset(self):
        self.wdt.reset()

    def test_disable(self):
        self.wdt.disable()

    def test_config_out_of_range(self):
        with pytest.raises(PSPInvalid):
            self.wdt.config(256)
        with pytest.raises(PSPInvalid):
            self.wdt.config(10, 3)

    def test_integration(self):
        with pytest.raises(PSPError):
            self.wdt.reset()
        self.wdt.enable(200)
        sleep(DELAY_TIME)
        self.wdt.reset()
        sleep(DELAY_TIME)
        self.wdt.disable()



# Configuration file for the Sphinx documentation builder.
#
# For the full list of built-in configuration values, see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

import os.path
import sys
from datetime import datetime
from pathlib import Path

import toml

# sys.path.insert(0, os.path.abspath('..'))
sys.path.append(os.path.join(os.path.dirname(__file__), '..'))
config = toml.load(Path(__file__).parent / '..' / 'pyproject.toml')
info = config['project']
from lannerpsp import __version__
info['version'] = __version__

# -- Project information -----------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#project-information

project = info['name']
author = info['authors'][0]['name']
project_copyright = f'{datetime.now():%Y}, Lanner Electronics Inc'
version = info['version']
release = info['version']

# -- General configuration ---------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#general-configuration

extensions = ['sphinx.ext.autodoc', 'sphinx.ext.viewcode', 'sphinx.ext.intersphinx']

source_suffix = '.rst'
# source_encoding = 'utf-8-sig'
exclude_patterns = ['_build']
templates_path = ['_templates']
# default_role = None
# keep_warnings = False
# today_fmt = '%B %d, %Y'
highlight_language = 'python3'
pygments_style = 'sphinx'
# add_function_parentheses = True
# add_module_names = True
# show_authors = False

# -- Autodoc configuration ---------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/extensions/autodoc.html

autodoc_member_order = 'bysource'

# -- Intersphinx configuration -----------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/extensions/intersphinx.html

intersphinx_mapping = {
    'python': ('https://docs.python.org/3', None),
}

# -- Options for internationalization ----------------------------------------

# language = 'zh_TW'

# -- Options for HTML output -------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#options-for-html-output

html_theme = 'sphinx_rtd_theme'
# html_theme_path = []
html_title = f'{info["name"]} {info["version"]} Documentation'
# html_short_title = None
html_logo = 'logo.svg'
html_favicon = 'favicon.ico'
html_static_path = ['_static']
# html_extra_path = []
# html_last_updated_fmt = '%b %d, %Y'
# html_use_smartypants = True
# html_additional_pages = {}
# html_domain_indices = True
# html_use_index = True
# html_split_index = False
# html_show_sourcelink = True
# html_use_opensearch = ''
# html_file_suffix = None
# html_show_copyright = True
# html_show_sphinx = True



"""
Platform: V3S
PSP version: 2.1.0
OS version:
- CentOS 7: All pass
SDK:
- sdk_can: Not implement yet, don't know how to use this SDK.
- sdk_dll: Done.
- sdk_gpio: Not implement yet, don't know how to use this SDK.
- sdk_gps: Only completed the `search port` function.
- sdk_gsr: Done, except `-set [register] [value]` and `-get [register]`, don't know how to use these functions.
- sdk_hwm: Done.
- sdk_ign: Not implement yet, don't know how to use this SDK.
- sdk_poe: Not implement yet, don't know how to use this SDK.
- sdk_wdt: Done.
"""
from time import sleep

import pytest

from lannerpsp import *

DELAY_TIME = 2  # Seconds.


class TestPSP:

    def test_init(self):
        with PSP():
            pass


class TestDLL:
    dll = DLL()

    def test_get_version(self):
        version = self.dll.get_version()
        assert version.dll_major == 2
        assert version.dll_minor == 1
        assert version.dll_build == 0
        assert version.platform_id == "V3S"
        assert version.board_major == 1
        assert version.board_minor == 0
        assert version.board_build == 2

    def test_get_bios_id(self):
        assert self.dll.get_bios_id() == 'V3S BIOS V1.x0 "12/05/2019"'


class TestGPS:
    gps = GPS()

    def test_search_port(self):
        port = self.gps.search_port()
        assert port == "/dev/ttyS3"


class TestGSR:
    gsr = GSR()

    def test_get_data(self):
        gsr_data = self.gsr.get_data()
        assert gsr_data.g_range == 2
        assert isinstance(gsr_data.mg_x, float)
        assert isinstance(gsr_data.mg_y, float)
        assert isinstance(gsr_data.mg_z, float)

    def test_get_offset(self):
        gsr_offset = self.gsr.get_offset()
        assert gsr_offset.raw_x == 0
        assert gsr_offset.raw_y == 0
        assert gsr_offset.raw_z == 0


class TestHWM:
    hwm = HWM()

    def test_list_supported_sensors(self):
        supported_sensors = self.hwm.list_supported_sensors()
        assert sorted([s.sid for s in supported_sensors]) == sorted([0, 2, 4, 7, 8, 11, 12])
        assert supported_sensors[0].sid == 0
        assert supported_sensors[0].name == "HWMID_TEMP_CPU1"
        assert supported_sensors[1].sid == 2
        assert supported_sensors[1].name == "HWMID_TEMP_SYS1"
        assert supported_sensors[2].sid == 4
        assert supported_sensors[2].name == "HWMID_VCORE_CPU1"
        assert supported_sensors[3].sid == 7
        assert supported_sensors[3].name == "HWMID_VOLT_P5V"
        assert supported_sensors[4].sid == 8
        assert supported_sensors[4].name == "HWMID_VOLT_P3V3"
        assert supported_sensors[5].sid == 11
        assert supported_sensors[5].name == "HWMID_VOLT_VBAT"
        assert supported_sensors[6].sid == 12
        assert supported_sensors[6].name == "HWMID_VOLT_DDRCH1"


class TestWDT:
    wdt = WDT()

    def test_get_info(self):
        wdt_info = self.wdt.get_info()
        assert wdt_info.type == "SuperIO"
        assert wdt_info.max_count == 255
        assert wdt_info.is_minute_support is True

    def test_enable(self):
        self.wdt.enable(10, 2)

    def test_reset(self):
        # PSP version 2.1 has a bug in the WDT `reset` function
        # that stops the timer like `disable`.
        self.wdt.reset()

    def test_disable(self):
        self.wdt.disable()

    def test_config_out_of_range(self):
        with pytest.raises(PSPInvalid):
            self.wdt.config(256)
        with pytest.raises(PSPInvalid):
            self.wdt.config(10, 3)

    def test_integration(self):
        with pytest.raises(PSPError):
            self.wdt.reset()
        self.wdt.enable(200)
        sleep(DELAY_TIME)
        self.wdt.reset()
        sleep(DELAY_TIME)
        self.wdt.disable()



# python -m build
# twine upload dist/*
"""
==========================
Python API for Lanner PSP.
==========================
"""
from .core import PSP, get_psp_exc_msg
from .exc import (
    IPMIError,
    IPMIIBF0,
    IPMIIdleState,
    IPMIOBF1,
    IPMIReadState,
    IPMIWriteState,
    PSPBoardNotMatch,
    PSPBusyInUses,
    PSPDriverNotLoad,
    PSPError,
    PSPInvalid,
    PSPNotExist,
    PSPNotOpened,
    PSPNotSupport,
    PSPWarning,
)
from .sdk_dll import DLL, DLLVersionModel
from .sdk_gps import GPS
from .sdk_gsr import GSR, GSRDataModel, GSROffsetModel
from .sdk_hwm import HWM, HWMSensorModel
from .sdk_lcm import LCM
from .sdk_odm_com_port import COMPort, COMPortInfoModel
from .sdk_rfm import RFM
from .sdk_sled import SystemLED
from .sdk_sled_gps import GPSStatusLED
from .sdk_sled_lte import LTEStatusLED
from .sdk_sled_lte_stress import LTEStressLED
from .sdk_swr import SWR
from .sdk_wdt import WDT, WDTInfoModel

__version__ = "0.0.10"
__all__ = [
    # Functions
    "get_psp_exc_msg",
    # Classes
    "COMPort",
    "DLL",
    "GPS",
    "GPSStatusLED",
    "GSR",
    "HWM",
    "LCM",
    "LTEStatusLED",
    "LTEStressLED",
    "PSP",
    "RFM",
    "SWR",
    "SystemLED",
    "WDT",
    # Models
    "COMPortInfoModel",
    "DLLVersionModel",
    "GSRDataModel",
    "GSROffsetModel",
    "HWMSensorModel",
    "WDTInfoModel",
    # Exceptions & Warnings
    "IPMIError",
    "IPMIIBF0",
    "IPMIIdleState",
    "IPMIOBF1",
    "IPMIReadState",
    "IPMIWriteState",
    "PSPBoardNotMatch",
    "PSPBusyInUses",
    "PSPDriverNotLoad",
    "PSPError",
    "PSPInvalid",
    "PSPNotExist",
    "PSPNotOpened",
    "PSPNotSupport",
    "PSPWarning",
]



import logging

from portio import inb, ioperm, outb

from .exc import PSPInvalid
from .utils import is_root

logger = logging.getLogger(__name__)


class GPIOConfigTool:
    """
    GPIO config tool for LEC-7242.
    """

    SIO_UART5_DEVICE = 0x14
    SIO_UART_EN = 0x30

    SIO_GPIO_DEVICE = 0x06
    SIO_GPIO_EN = 0x30
    REG_GPIO0_IRQ_SEL = 0x70
    REG_GPIO_IRQ_EN = 0x7E
    REG_GPIO_IRQ_MODE = 0x7F

    REG_GPIO0_OUT_EN = 0xF0
    REG_GPIO0_OUT_DATA = 0xF1
    REG_GPIO0_PIN_STAT = 0xF2
    REG_GPIO0_DRIVE = 0xF3
    REG_GPIO0_SMI_EN = 0xF8
    REG_GPIO0_SMI_STATUS = 0xF9

    REG_GPIO1_OUT_EN = 0xE0
    REG_GPIO1_OUT_DATA = 0xE1
    REG_GPIO1_PIN_STAT = 0xE2
    REG_GPIO1_DRIVE = 0xE3

    REG_GPIO2_OUT_EN = 0xD0
    REG_GPIO2_OUT_DATA = 0xD1
    REG_GPIO2_PIN_STAT = 0xD2
    REG_GPIO2_DRIVE = 0xD3

    SIO_INDEX = 0x2e
    SIO_DATA = 0x2f

    def __init__(self):
        pass

    def set_com1_mode(self, mode: int) -> None:
        """
        Set COM1 mode to RS-232, RS-422 or RS-485.

        Example:

        .. code-block:: pycon

            >>> GPIOConfigTool().set_com1_mode(232)

        :param int mode: 232/422/485
        :raises PermissionError: if not running as root user
        :raises TypeError: The input parameters type error.
        :raises PSPInvalid: The input parameter is out of range.
        """
        # Check permission.
        if not is_root():
            raise PermissionError("Please uses root user !!!")
        # Check type.
        if not isinstance(mode, int):
            raise TypeError("'mode' type must be int")
        # Set mode.
        if mode == 232:
            self._com3_switch_mode(1)
        elif mode == 422:
            self._com3_switch_mode(3)
        elif mode == 485:
            self._com3_switch_mode(2)
        else:
            raise PSPInvalid("'mode' value must be 232 or 422 or 485")
        logger.debug(f"set com1 mode {mode}")

    def set_com1_termination(self, enable: bool) -> None:
        """
        Set COM1 RS-422/RS-485 termination.

        Example:

        .. code-block:: pycon

            >>> GPIOConfigTool().set_com1_termination(True)

        :param bool enable: ``True`` = enable, ``False`` = disable
        :raises PermissionError: if not running as root user
        :raises TypeError: The input parameters type error.
        """
        # Check permission.
        if not is_root():
            raise PermissionError("Please uses root user !!!")
        # Check type.
        if not isinstance(enable, bool):
            raise TypeError("'enable' type must be bool")
        # Set termination.
        if enable:
            self._com3_term_ctrl(1)
        else:
            self._com3_term_ctrl(0)
        logger.debug(f"set com1 termination {enable}")

    def _sio_unlock(self) -> None:
        ioperm(self.SIO_INDEX, 2, 1)
        outb(0x87, self.SIO_INDEX)
        outb(0x87, self.SIO_INDEX)

    def _sio_lock(self) -> None:
        outb(0xaa, self.SIO_INDEX)
        ioperm(self.SIO_INDEX, 2, 0)

    def _com3_switch_mode(self, mode: int) -> None:
        """
        mode 0: loop back
        mode 1: rs232
        mode 2: rs485
        mode 3: rs422
        """
        self._sio_unlock()

        # set multi function into gpio1x (13/16)
        outb(0x27, self.SIO_INDEX)
        xch = (inb(self.SIO_DATA) & 0xF2) | 0x04
        outb(0x27, self.SIO_INDEX)
        outb(xch, self.SIO_DATA)

        outb(0x29, self.SIO_INDEX)
        xch = inb(self.SIO_DATA) & 0xF9
        outb(0x29, self.SIO_INDEX)
        outb(xch, self.SIO_DATA)

        outb(0x2c, self.SIO_INDEX)
        xch = inb(self.SIO_DATA) | 0x48
        outb(0x2c, self.SIO_INDEX)
        outb(xch, self.SIO_DATA)

        # set multi function into gpio2x (20)
        outb(0x27, self.SIO_INDEX)
        xch = (inb(self.SIO_DATA) & 0xF2) | 0x08
        outb(0x27, self.SIO_INDEX)
        outb(xch, self.SIO_DATA)

        outb(0x29, self.SIO_INDEX)
        xch = inb(self.SIO_DATA) & 0xF7
        outb(0x29, self.SIO_INDEX)
        outb(xch, self.SIO_DATA)

        outb(0x2c, self.SIO_INDEX)
        xch = inb(self.SIO_DATA) | 0x01
        outb(0x2c, self.SIO_INDEX)
        outb(xch, self.SIO_DATA)
        # print("set multi function into gpio")

        # enable GPIO logical device
        outb(0x07, self.SIO_INDEX)
        outb(self.SIO_GPIO_DEVICE, self.SIO_DATA)

        # active GPIO logic device
        outb(self.SIO_GPIO_EN, self.SIO_INDEX)
        xch = inb(self.SIO_DATA) | 0x01
        outb(self.SIO_GPIO_EN, self.SIO_INDEX)
        outb(xch, self.SIO_DATA)  # active GPIO
        # print("active gpio function")

        outb(self.REG_GPIO1_OUT_EN, self.SIO_INDEX)
        xch = inb(self.SIO_DATA) | 0x48  # set gpio13/gpio16 output pins
        outb(xch, self.SIO_DATA)

        outb(self.REG_GPIO2_OUT_EN, self.SIO_INDEX)
        xch = inb(self.SIO_DATA) | 0x01  # set gpio20 output pins
        outb(xch, self.SIO_DATA)

        outb(self.REG_GPIO1_OUT_DATA, self.SIO_INDEX)  # GPIO1 Output Data Register
        xch = inb(self.SIO_DATA) & 0xBF  # clear gpio16 output pins
        xch |= (mode << 5)  # set gpio16 output value
        xch |= 0x08  # set gpio13 output 1
        outb(xch, self.SIO_DATA)

        outb(self.REG_GPIO2_OUT_DATA, self.SIO_INDEX)  # GPIO2 Output Data Register
        xch = inb(self.SIO_DATA) & 0xFE  # clear gpio20 output pins
        xch |= mode  # set gpio20 output value
        outb(xch, self.SIO_DATA)

        self._sio_lock()

    def _com3_term_ctrl(self, on_off: int) -> None:
        self._sio_unlock()

        # enable GPIO logical device
        outb(0x07, self.SIO_INDEX)
        outb(self.SIO_GPIO_DEVICE, self.SIO_DATA)

        outb(self.REG_GPIO1_OUT_DATA, self.SIO_INDEX)  # GPIO1 Output Data Register
        xch = inb(self.SIO_DATA) & 0xF7  # clear gpio13 output pins
        if on_off:
            xch |= 0x08  # set gpio13 output pins, enable term
        outb(xch, self.SIO_DATA)

        self._sio_lock()



class PSPError(Exception):
    """Generic error message (Base class for all exceptions in Lanner PSP)"""


class PSPNotExist(PSPError, FileNotFoundError):
    """Error raised when library file is not found/plugged, or the device does not exist"""


class PSPNotOpened(PSPError, RuntimeError):
    """Error raised when library is not opened or not ready yet"""


class PSPInvalid(PSPError, ValueError):
    """Error raised when the Parameter is not a valid value or out of range"""


class PSPNotSupport(PSPError, RuntimeError):
    """Error raised when this function is not supported in this platform"""


class PSPBusyInUses(PSPError, RuntimeError):
    """Error raised when library is being used now or device IO is busy now"""


class PSPBoardNotMatch(PSPError, RuntimeError):
    """Error raised when the board library and the board do not match"""


class PSPDriverNotLoad(PSPError, RuntimeError):
    """Error raised when the lmbiodrv driver or i2c-dev driver not loading"""


class IPMIError(PSPError):
    """Base class for errors related to the IPMI implementation"""


class IPMIIdleState(IPMIError, RuntimeError):
    """Error raised when IPMI KCS interface is not in IDLE State"""


class IPMIWriteState(IPMIError, RuntimeError):
    """Error raised when IPMI KCS interface WRITE State check failure"""


class IPMIReadState(IPMIError, RuntimeError):
    """Error raised when IPMI KCS interface READ State check failure"""


class IPMIIBF0(IPMIError, RuntimeError):
    """Error raised when IPMI KCS interface wait IBF ‘0’ State failure"""


class IPMIOBF1(IPMIError, RuntimeError):
    """Error raised when IPMI KCS interface wait OBF ‘1’ State failure"""


class PSPWarning(Warning):
    """Base class for all warnings in Lanner PSP"""



import logging
from ctypes import byref, c_char_p, c_int32, c_uint8, CFUNCTYPE, sizeof

from .core import PSP, get_psp_exc_msg
from .exc import (
    PSPError,
    PSPNotExist,
    PSPInvalid,
    PSPNotOpened,
    PSPNotSupport,
)
from .lmbinc import (
    ERR_Invalid,
    ERR_NotExist,
    ERR_NotOpened,
    ERR_NotSupport,
    ERR_Success,
    LCMInfo,
    LCMKeyMsg,
)
from .sdk_dll import DLL

logger = logging.getLogger(__name__)

SUPPORTED_PLATFORMS = ("NCA-2510",)
UNSUPPORTED_PLATFORMS = ("LEB-7242", "LEC-7230", "V3S", "V6S",)

DEFAULT_LCM_PORT = "/dev/ttyS1"
DEFAULT_BAUD_RATE = 19200

LCM_UART_TYPE = 1
LCM_LPT_TYPE = 2


class LCM:
    """
    LCD Module.

    :param bool check_platform:
        Set to :data:`True` to check if the platform supports this feature.
        Defaults to :data:`False` for better compatibility.
    """

    def __init__(self, check_platform: bool = False) -> None:
        self._str_lcm_port = c_char_p(DEFAULT_LCM_PORT.encode())
        self._dw_speed = DEFAULT_BAUD_RATE
        self._version = DLL().get_version()
        if not check_platform:
            return
        if self._version.platform_id in SUPPORTED_PLATFORMS:
            pass
        elif self._version.platform_id in UNSUPPORTED_PLATFORMS:
            raise PSPNotSupport("Not supported on this platform")
        else:
            raise NotImplementedError

    def search_port(self) -> str:
        """
        Get the LCM current connected port and speed.

        :return: the port and speed the LCM is currently connected to
        :rtype: str
        :raises PSPNotOpened: The library is not ready or opened yet.
        :raises PSPNotSupport: This function is not supported.
        :raises PSPNotExist: This function is not enabled or does not exist.
        :raises PSPError: General PSP functional error.
        """
        # TODO: Example
        dw_speed = c_int32(self._dw_speed)
        with PSP() as psp:
            if self._version.dll_major == 2 and self._version.dll_minor in (1, 2, 3):
                i_ret = psp.lib.LMB_LCM_SearchPort(self._str_lcm_port, byref(dw_speed))
            elif self._version.dll_major == 3 and self._version.dll_minor in (0,):
                i_ret = psp.lib.LMB_LCM_SearchPort(self._str_lcm_port, byref(dw_speed), sizeof(self._str_lcm_port))
            else:
                raise NotImplementedError
        msg = get_psp_exc_msg("LMB_LCM_SearchPort", i_ret)
        if i_ret == ERR_Success:
            return f"port={self._str_lcm_port}, speed={dw_speed.value:d}"
        elif i_ret == ERR_NotOpened:
            raise PSPNotOpened(msg)
        elif i_ret == ERR_NotSupport:
            raise PSPNotSupport(msg)
        elif i_ret == ERR_NotExist:
            raise PSPNotExist(msg)
        else:
            raise PSPError(msg)

    def _open_port(self, psp: PSP) -> None:
        """
        Open the LCM device with path and assigned speed.

        .. Note::
            * Support only **UART** type.
            * The string maximum length depends on LCD module, normally 40 characters.

        :raises PSPNotOpened: The library is not ready or opened yet.
        :raises PSPNotSupport: This function is not supported.
        :raises PSPNotExist: This function is not enabled or does not exist.
        :raises PSPError: General PSP functional error.
        """
        i_ret = psp.lib.LMB_LCM_OpenPort(self._str_lcm_port, self._dw_speed)
        msg = get_psp_exc_msg("LMB_LCM_OpenPort", i_ret)
        if i_ret == ERR_Success:
            pass
        elif i_ret == ERR_NotOpened:
            raise PSPNotOpened(msg)
        elif i_ret == ERR_NotSupport:
            raise PSPNotSupport(msg)
        elif i_ret == ERR_NotExist:
            raise PSPNotExist(msg)
        else:
            raise PSPError(msg)

    def _open_device(self, psp: PSP) -> None:
        """
        Open and connect LCD module.

        :raises PSPNotOpened: The library is not ready or opened yet.
        :raises PSPNotSupport: This function is not supported.
        :raises PSPNotExist: This function is not enabled or does not exist.
        :raises PSPError: General PSP functional error.
        """
        i_ret = psp.lib.LMB_LCM_DeviceOpen()
        msg = get_psp_exc_msg("LMB_LCM_DeviceOpen", i_ret)
        if i_ret == ERR_Success:
            pass
        elif i_ret == ERR_NotOpened:
            raise PSPNotOpened(msg)
        elif i_ret == ERR_NotSupport:
            raise PSPNotSupport(msg)
        elif i_ret == ERR_NotExist:
            raise PSPNotExist(msg)
        else:
            raise PSPError(msg)

    def _close_device(self, psp: PSP) -> None:
        """
        Close and disconnect LCD module.

        :raises PSPNotOpened: The library is not ready or opened yet.
        :raises PSPNotSupport: This function is not supported.
        :raises PSPError: General PSP functional error.
        """
        i_ret = psp.lib.LMB_LCM_DeviceClose()
        msg = get_psp_exc_msg("LMB_LCM_DeviceClose", i_ret)
        if i_ret == ERR_Success:
            pass
        elif i_ret == ERR_NotOpened:
            raise PSPNotOpened(msg)
        elif i_ret == ERR_NotSupport:
            raise PSPNotSupport(msg)
        else:
            raise PSPError(msg)

    def _get_device_info(self, psp: PSP) -> int:
        """
        Get platform LCM device information.

        :return: LCM module type (UART = 1, LPT = 2)
        :rtype: int
        """
        stu_lcm_info = LCMInfo()
        i_ret = psp.lib.LMB_LCM_DeviceInfo(byref(stu_lcm_info))
        msg = get_psp_exc_msg("LMB_LCM_DeviceInfo", i_ret)
        if i_ret == ERR_Success:
            logger.debug(f"LCM Mode No. is {stu_lcm_info.uw_mode_no:04X}\n"
                         f"LCM Firmware Ver. is {stu_lcm_info.uw_version:04X}\n"
                         f"LCM Speed ={stu_lcm_info.udw_baud_rate:d}")
            return LCM_UART_TYPE
        else:
            # ERR_NotOpened or ERR_NotSupport.
            logger.debug(msg)
            return LCM_LPT_TYPE

    def reset(self) -> None:
        """
        Reset the LCD module.

        .. Note::
            Support only **UART** type.

        :raises PSPNotOpened: The library is not ready or opened yet.
        :raises PSPNotSupport: This function is not supported.
        :raises PSPError: General PSP functional error.
        """
        # TODO: Example
        with PSP() as psp:
            self._open_port(psp)
            if self._get_device_info(psp) == LCM_LPT_TYPE:
                raise PSPNotSupport("LPT type not support reset")
            i_ret = psp.lib.LMB_LCM_Reset()
            msg = get_psp_exc_msg("LMB_LCM_Reset", i_ret)
            if i_ret == ERR_Success:
                pass
            elif i_ret == ERR_NotOpened:
                raise PSPNotOpened(msg)
            elif i_ret == ERR_NotSupport:
                raise PSPNotSupport(msg)
            else:
                raise PSPError(msg)
            self._close_device(psp)

    def set_backlight(self, enable: bool) -> None:
        """
        Set LCD module light on/off status.

        Example:

        .. code-block:: pycon

            >>> lcm = LCM()
            >>> lcm.set_backlight(True)

        :param bool enable: set :data:`True` to enable backlight, otherwise :data:`False`
        :raises TypeError: The input parameters type error.
        :raises PSPNotOpened: The library is not ready or opened yet.
        :raises PSPNotSupport: This function is not supported.
        :raises PSPError: General PSP functional error.
        """
        # Check type.
        if not isinstance(enable, bool):
            raise TypeError("'enable' type must be bool")
        # Run.
        with PSP() as psp:
            self._open_device(psp)
            i_ret = psp.lib.LMB_LCM_LightCtrl(c_uint8(enable & 0xFF).value)
            msg = get_psp_exc_msg("LMB_LCM_LightCtrl", i_ret)
            if i_ret == ERR_Success:
                logger.debug(f"set LCM backlight to {enable}")
            elif i_ret == ERR_NotOpened:
                raise PSPNotOpened(msg)
            elif i_ret == ERR_NotSupport:
                raise PSPNotSupport(msg)
            else:
                raise PSPError(msg)
            self._close_device(psp)

    def set_cursor(self, row: int, column: int = 1) -> None:
        """
        Set LCM cursor.

        Example:

        .. code-block:: pycon

            >>> lcm = LCM()
            >>> lcm.set_cursor(2, 6)

        :param int row: row value of LCM display cursor
        :param int column: column value of LCM display cursor
        :raises TypeError: The input parameters type error.
        :raises PSPInvalid: The input parameter is out of range.
        :raises PSPNotOpened: The library is not ready or opened yet.
        :raises PSPNotSupport: This function is not supported.
        :raises PSPError: General PSP functional error.
        """
        # Check type.
        if not isinstance(row, int):
            raise TypeError("'row' type must be int")
        if not isinstance(column, int):
            raise TypeError("'column' type must be int")
        # Check value has been done by the PSP.
        # Run.
        with PSP() as psp:
            self._open_device(psp)
            i_ret = psp.lib.LMB_LCM_SetCursor(column, row)
            msg = get_psp_exc_msg("LMB_LCM_SetCursor", i_ret)
            if i_ret == ERR_Success:
                logger.debug(f"set LCM cursor to row {row} column {column}")
            elif i_ret == ERR_Invalid:
                raise PSPInvalid(msg)
            elif i_ret == ERR_NotOpened:
                raise PSPNotOpened(msg)
            elif i_ret == ERR_NotSupport:
                raise PSPNotSupport(msg)
            else:
                raise PSPError(msg)
            self._close_device(psp)

    def write(self, msg: str) -> None:
        """
        Write string to LCD module.

        .. Note::
            The maximum string length depends on the LCD module, normally 40 characters.

        Example:

        .. code-block:: pycon

            >>> lcm = LCM()
            >>> lcm.write("Hello World")

        :param str msg: message string to LCM display
        :raises TypeError: The input parameters type error.
        :raises PSPNotOpened: The library is not ready or opened yet.
        :raises PSPNotSupport: This function is not supported.
        :raises PSPError: General PSP functional error.
        """
        # Check type.
        if not isinstance(msg, str):
            raise TypeError("'msg' type must be str")
        # Run.
        with PSP() as psp:
            self._open_device(psp)
            i_ret = psp.lib.LMB_LCM_WriteString(c_char_p(msg.encode()))
            msg = get_psp_exc_msg("LMB_LCM_WriteString", i_ret)
            if i_ret == ERR_Success:
                logger.debug(f"write '{msg}' on LCM")
            elif i_ret == ERR_NotOpened:
                raise PSPNotOpened(msg)
            elif i_ret == ERR_NotSupport:
                raise PSPNotSupport(msg)
            else:
                raise PSPError(msg)
            self._close_device(psp)

    def clear(self) -> None:
        """
        Clear LCM display.

        Example:

        .. code-block:: pycon

            >>> lcm = LCM()
            >>> lcm.clear()

        :raises PSPNotOpened: The library is not ready or opened yet.
        :raises PSPNotSupport: This function is not supported.
        :raises PSPError: General PSP functional error.
        """
        with PSP() as psp:
            self._open_device(psp)
            i_ret = psp.lib.LMB_LCM_DisplayClear()
            msg = get_psp_exc_msg("LMB_LCM_DisplayClear", i_ret)
            if i_ret == ERR_Success:
                logger.debug(f"clear string on LCM")
            elif i_ret == ERR_NotOpened:
                raise PSPNotOpened(msg)
            elif i_ret == ERR_NotSupport:
                raise PSPNotSupport(msg)
            else:
                raise PSPError(msg)
            self._close_device(psp)

    def get_keys_status(self) -> int:
        """
        Get LCM keys status.

        bit 0 means Key 1, bit 1 means Key2, bit 2 means Key 3, bit 3 means Key4

        1: pressed , 0: released

        - 0 (0000): key1 -> off, key2 -> off, key3 -> off, key4 -> off
        - 1 (0001): key1 -> on, key2 -> off, key3 -> off, key4 -> off
        - 2 (0010): key1 -> off, key2 -> on, key3 -> off, key4 -> off
        - 4 (0100): key1 -> off, key2 -> off, key3 -> on, key4 -> off
        - 8 (1000): key1 -> off, key2 -> off, key3 -> off, key4 -> on

        Example:

        .. code-block:: pycon

            >>> lcm = LCM()
            >>> lcm.get_keys_status()
            2

        :return: LCM keys status
        :rtype: int
        :raises PSPNotOpened: The library is not ready or opened yet.
        :raises PSPNotSupport: This function is not supported.
        :raises PSPError: No key is pressed.
        """
        ub_keys = c_uint8()
        with PSP() as psp:
            self._open_device(psp)
            i_ret = psp.lib.LMB_LCM_KeysStatus(byref(ub_keys))
            msg = get_psp_exc_msg("LMB_LCM_KeysStatus", i_ret)
            if i_ret == ERR_Success:
                logger.debug(f"LCM keys status is {ub_keys.value:02x}")
            elif i_ret == ERR_NotOpened:
                raise PSPNotOpened(msg)
            elif i_ret == ERR_NotSupport:
                raise PSPNotSupport(msg)
            else:
                raise PSPError(msg)
            self._close_device(psp)
            return ub_keys.value

    @classmethod
    def _callback(cls, stu_lcm_msg: LCMKeyMsg) -> None:
        """Callback function for :func:`exec_callback`."""
        print(f"<Callback> LCM Item = 0x{stu_lcm_msg.ub_keys:02X}, "
              f"Status = 0x{stu_lcm_msg.ub_status:02X}, "
              f"time is {stu_lcm_msg.stu_time.uw_year:04d}/"
              f"{stu_lcm_msg.stu_time.ub_month:02d}/"
              f"{stu_lcm_msg.stu_time.ub_day:02d} "
              f"{stu_lcm_msg.stu_time.ub_hour:02d}:"
              f"{stu_lcm_msg.stu_time.ub_minute:02d}:"
              f"{stu_lcm_msg.stu_time.ub_second:02d}")

    def exec_callback(self) -> None:
        """
        Use callback function to detect LCM Keys status.

        Example:

        .. code-block:: pycon

            >>> lcm = LCM()
            >>> lcm.exec_callback()
            ----> hook LCM Keys Callback OK <----
            ===> pause !!! hit <enter> to end <===
            <Callback> LCM Item = 0x08, Status = 0x08, time is 2022/08/02 16:48:28
            <Callback> LCM Item = 0x08, Status = 0x00, time is 2022/08/02 16:48:28
            <Callback> LCM Item = 0x04, Status = 0x04, time is 2022/08/02 16:48:29
            <Callback> LCM Item = 0x04, Status = 0x00, time is 2022/08/02 16:48:29
            <Callback> LCM Item = 0x08, Status = 0x08, time is 2022/08/02 16:48:30
            <Callback> LCM Item = 0x08, Status = 0x00, time is 2022/08/02 16:48:30
            <Callback> LCM Item = 0x02, Status = 0x02, time is 2022/08/02 16:48:31
            <Callback> LCM Item = 0x02, Status = 0x00, time is 2022/08/02 16:48:31
            <Callback> LCM Item = 0x01, Status = 0x01, time is 2022/08/02 16:48:31
            <Callback> LCM Item = 0x01, Status = 0x00, time is 2022/08/02 16:48:31
            <Callback> LCM Item = 0x08, Status = 0x08, time is 2022/08/02 16:48:32
            <Callback> LCM Item = 0x08, Status = 0x00, time is 2022/08/02 16:48:32

            ----> hook LCM Keys Callback Disable OK <----
        """
        c_callback = CFUNCTYPE(None, LCMKeyMsg)
        p_callback = c_callback(self._callback)
        with PSP() as psp:
            self._open_device(psp)
            i_ret = psp.lib.LMB_LCM_KeysCallback(p_callback, 150)
            if i_ret != ERR_Success:
                print("-----> hook LCM Keys callback failure <-------")
                return
            print("----> hook LCM Keys Callback OK <----")
            print("===> pause !!! hit <enter> to end <===")
            input()
            i_ret = psp.lib.LMB_LCM_KeysCallback(None, 150)
            if i_ret == ERR_Success:
                print("----> hook LCM Keys Callback Disable OK <----")
            self._close_device(psp)



import logging

from .core import PSP, get_psp_exc_msg
from .exc import (
    PSPError,
    PSPInvalid,
    PSPNotOpened,
    PSPNotSupport,
)
from .lmbinc import (
    ERR_Invalid,
    ERR_NotOpened,
    ERR_NotSupport,
    ERR_Success,
)
from .sdk_dll import DLL
from .utils import show_delay

logger = logging.getLogger(__name__)

SUPPORTED_PLATFORMS = ("LEB-7242",)
UNSUPPORTED_PLATFORMS = ("LEC-7230", "NCA-2510", "V3S", "V6S",)


class LTEStatusLED:
    """
    LTE Status LED.

    :param bool check_platform:
        Set to :data:`True` to check if the platform supports this feature.
        Defaults to :data:`False` for better compatibility.
    """

    def __init__(self, check_platform: bool = False) -> None:
        self._version = DLL().get_version()
        if not check_platform:
            return
        if self._version.platform_id in SUPPORTED_PLATFORMS:
            pass
        elif self._version.platform_id in UNSUPPORTED_PLATFORMS:
            raise PSPNotSupport("Not supported on this platform")
        else:
            raise NotImplementedError

    def off(self) -> None:
        """
        Set the LTE status LED display mode to off.

        Example:

        .. code-block:: pycon

            >>> lte_status_led = LTEStatusLED()
            >>> lte_status_led.off()

        :raises PSPNotOpened: The library is not ready or opened yet.
        :raises PSPInvalid: The input parameter is out of range.
        :raises PSPNotSupport: This function is not supported.
        :raises PSPError: General PSP functional error.
        """
        with PSP() as psp:
            i_ret = psp.lib.LMB_SLED_SetLteStateLED(0)
        msg = get_psp_exc_msg("LMB_SLED_SetLteStateLED", i_ret)
        if i_ret == ERR_Success:
            logger.debug("set lte led off")
        elif i_ret == ERR_NotOpened:
            raise PSPNotOpened(msg)
        elif i_ret == ERR_Invalid:
            raise PSPInvalid(msg)
        elif i_ret == ERR_NotSupport:
            raise PSPNotSupport(msg)
        else:
            raise PSPError(msg)

    def red(self) -> None:
        """
        Set the LTE status LED display mode to red.

        Example:

        .. code-block:: pycon

            >>> lte_status_led = LTEStatusLED()
            >>> lte_status_led.red()

        :raises PSPNotOpened: The library is not ready or opened yet.
        :raises PSPInvalid: The input parameter is out of range.
        :raises PSPNotSupport: This function is not supported.
        :raises PSPError: General PSP functional error.
        """
        self.off()  # Clear color.
        with PSP() as psp:
            i_ret = psp.lib.LMB_SLED_SetLteStateLED(1)
        msg = get_psp_exc_msg("LMB_SLED_SetLteStateLED", i_ret)
        if i_ret == ERR_Success:
            logger.debug("set lte led red on")
        elif i_ret == ERR_NotOpened:
            raise PSPNotOpened(msg)
        elif i_ret == ERR_Invalid:
            raise PSPInvalid(msg)
        elif i_ret == ERR_NotSupport:
            raise PSPNotSupport(msg)
        else:
            raise PSPError(msg)

    def red_blink(self) -> None:
        """
        Set the LTE status LED display mode to red blink.

        Example:

        .. code-block:: pycon

            >>> lte_status_led = LTEStatusLED()
            >>> lte_status_led.red_blink()

        :raises PSPNotOpened: The library is not ready or opened yet.
        :raises PSPInvalid: The input parameter is out of range.
        :raises PSPNotSupport: This function is not supported.
        :raises PSPError: General PSP functional error.
        """
        self.off()  # Clear color.
        with PSP() as psp:
            i_ret = psp.lib.LMB_SLED_SetLteStateLED(2)
        msg = get_psp_exc_msg("LMB_SLED_SetLteStateLED", i_ret)
        if i_ret == ERR_Success:
            logger.debug("set lte led red blink")
        elif i_ret == ERR_NotOpened:
            raise PSPNotOpened(msg)
        elif i_ret == ERR_Invalid:
            raise PSPInvalid(msg)
        elif i_ret == ERR_NotSupport:
            raise PSPNotSupport(msg)
        else:
            raise PSPError(msg)

    def green(self) -> None:
        """
        Set the LTE status LED display mode to green.

        Example:

        .. code-block:: pycon

            >>> lte_status_led = LTEStatusLED()
            >>> lte_status_led.green()

        :raises PSPNotOpened: The library is not ready or opened yet.
        :raises PSPInvalid: The input parameter is out of range.
        :raises PSPNotSupport: This function is not supported.
        :raises PSPError: General PSP functional error.
        """
        self.off()  # Clear color.
        with PSP() as psp:
            i_ret = psp.lib.LMB_SLED_SetLteStateLED(3)
        msg = get_psp_exc_msg("LMB_SLED_SetLteStateLED", i_ret)
        if i_ret == ERR_Success:
            logger.debug("set lte led green on")
        elif i_ret == ERR_NotOpened:
            raise PSPNotOpened(msg)
        elif i_ret == ERR_Invalid:
            raise PSPInvalid(msg)
        elif i_ret == ERR_NotSupport:
            raise PSPNotSupport(msg)
        else:
            raise PSPError(msg)

    def green_blink(self) -> None:
        """
        Set the LTE status LED display mode to green blink.

        Example:

        .. code-block:: pycon

            >>> lte_status_led = LTEStatusLED()
            >>> lte_status_led.green_blink()

        :raises PSPNotOpened: The library is not ready or opened yet.
        :raises PSPInvalid: The input parameter is out of range.
        :raises PSPNotSupport: This function is not supported.
        :raises PSPError: General PSP functional error.
        """
        self.off()  # Clear color.
        with PSP() as psp:
            i_ret = psp.lib.LMB_SLED_SetLteStateLED(4)
        msg = get_psp_exc_msg("LMB_SLED_SetLteStateLED", i_ret)
        if i_ret == ERR_Success:
            logger.debug("set lte led green blink")
        elif i_ret == ERR_NotOpened:
            raise PSPNotOpened(msg)
        elif i_ret == ERR_Invalid:
            raise PSPInvalid(msg)
        elif i_ret == ERR_NotSupport:
            raise PSPNotSupport(msg)
        else:
            raise PSPError(msg)

    def yellow(self) -> None:
        """
        Set the LTE status LED display mode to yellow.

        Example:

        .. code-block:: pycon

            >>> lte_status_led = LTEStatusLED()
            >>> lte_status_led.yellow()

        :raises PSPNotOpened: The library is not ready or opened yet.
        :raises PSPInvalid: The input parameter is out of range.
        :raises PSPNotSupport: This function is not supported.
        :raises PSPError: General PSP functional error.
        """
        self.off()  # Clear color.
        with PSP() as psp:
            i_ret = psp.lib.LMB_SLED_SetLteStateLED(5)
        msg = get_psp_exc_msg("LMB_SLED_SetLteStateLED", i_ret)
        if i_ret == ERR_Success:
            logger.debug("set lte led yellow on")
        elif i_ret == ERR_NotOpened:
            raise PSPNotOpened(msg)
        elif i_ret == ERR_Invalid:
            raise PSPInvalid(msg)
        elif i_ret == ERR_NotSupport:
            raise PSPNotSupport(msg)
        else:
            raise PSPError(msg)

    def yellow_blink(self) -> None:
        """
        Set the LTE status LED display mode to yellow blink.

        Example:

        .. code-block:: pycon

            >>> lte_status_led = LTEStatusLED()
            >>> lte_status_led.yellow_blink()

        :raises PSPNotOpened: The library is not ready or opened yet.
        :raises PSPInvalid: The input parameter is out of range.
        :raises PSPNotSupport: This function is not supported.
        :raises PSPError: General PSP functional error.
        """
        self.off()  # Clear color.
        with PSP() as psp:
            i_ret = psp.lib.LMB_SLED_SetLteStateLED(6)
        msg = get_psp_exc_msg("LMB_SLED_SetLteStateLED", i_ret)
        if i_ret == ERR_Success:
            logger.debug("set lte led yellow blink")
        elif i_ret == ERR_NotOpened:
            raise PSPNotOpened(msg)
        elif i_ret == ERR_Invalid:
            raise PSPInvalid(msg)
        elif i_ret == ERR_NotSupport:
            raise PSPNotSupport(msg)
        else:
            raise PSPError(msg)

    def test(self, secs: int = 2) -> None:
        """
        For testing (default 2 seconds delay).

        Example:

        .. code-block:: pycon

            >>> lte_status_led = LTEStatusLED()
            >>> lte_status_led.test()
            set lte led red on
            2. 1. 0.
            set lte led red blink
            2. 1. 0.
            set lte led off
            2. 1. 0.
            set lte led green on
            2. 1. 0.
            set lte led green blink
            2. 1. 0.
            set lte led off
            2. 1. 0.
            set lte led yellow on
            2. 1. 0.
            set lte led yellow blink
            2. 1. 0.
            set lte led off

        :param int secs: seconds for test
        :raises TypeError: The input parameters type error.
        :raises ValueError: The input parameters value error.
        """
        # Check type.
        if not isinstance(secs, int):
            raise TypeError("'secs' type must be int")
        # Check value.
        if secs <= 0:
            raise ValueError("'secs' value must be >= 0")
        # Run.
        with PSP() as psp:
            i_ret = psp.lib.LMB_SLED_SetLteStateLED(1)
            if i_ret != ERR_Success:
                msg = get_psp_exc_msg("LMB_SLED_SetLteStateLED", i_ret)
                print(f"\033[1;31m{msg}\033[0m")
            else:
                print("set lte led red on")

            show_delay(secs)

            i_ret = psp.lib.LMB_SLED_SetLteStateLED(2)
            if i_ret != ERR_Success:
                msg = get_psp_exc_msg("LMB_SLED_SetLteStateLED", i_ret)
                print(f"\033[1;31m{msg}\033[0m")
            else:
                print("set lte led red blink")

            show_delay(secs)

            i_ret = psp.lib.LMB_SLED_SetLteStateLED(0)
            if i_ret != ERR_Success:
                msg = get_psp_exc_msg("LMB_SLED_SetLteStateLED", i_ret)
                print(f"\033[1;31m{msg}\033[0m")
            else:
                print("set lte led off")

            show_delay(secs)

            i_ret = psp.lib.LMB_SLED_SetLteStateLED(3)
            if i_ret != ERR_Success:
                msg = get_psp_exc_msg("LMB_SLED_SetLteStateLED", i_ret)
                print(f"\033[1;31m{msg}\033[0m")
            else:
                print("set lte led green on")

            show_delay(secs)

            i_ret = psp.lib.LMB_SLED_SetLteStateLED(4)
            if i_ret != ERR_Success:
                msg = get_psp_exc_msg("LMB_SLED_SetLteStateLED", i_ret)
                print(f"\033[1;31m{msg}\033[0m")
            else:
                print("set lte led green blink")

            show_delay(secs)

            i_ret = psp.lib.LMB_SLED_SetLteStateLED(0)
            if i_ret != ERR_Success:
                msg = get_psp_exc_msg("LMB_SLED_SetLteStateLED", i_ret)
                print(f"\033[1;31m{msg}\033[0m")
            else:
                print("set lte led off")

            show_delay(secs)

            i_ret = psp.lib.LMB_SLED_SetLteStateLED(5)
            if i_ret != ERR_Success:
                msg = get_psp_exc_msg("LMB_SLED_SetLteStateLED", i_ret)
                print(f"\033[1;31m{msg}\033[0m")
            else:
                print("set lte led yellow on")

            show_delay(secs)

            i_ret = psp.lib.LMB_SLED_SetLteStateLED(6)
            if i_ret != ERR_Success:
                msg = get_psp_exc_msg("LMB_SLED_SetLteStateLED", i_ret)
                print(f"\033[1;31m{msg}\033[0m")
            else:
                print("set lte led yellow blink")

            show_delay(secs)

            i_ret = psp.lib.LMB_SLED_SetLteStateLED(0)
            if i_ret != ERR_Success:
                msg = get_psp_exc_msg("LMB_SLED_SetLteStateLED", i_ret)
                print(f"\033[1;31m{msg}\033[0m")
            else:
                print("set lte led off")



"""
Platform: LEC-7242
PSP version: 2.1.2
OS version:
- Debian 10: All pass
- Debian 11: All pass
SDK:
- sdk_bios: Done.
- sdk_dll: Done.
- sdk_hwm: Done.
- sdk_rfm: Done.
- sdk_sled: Done.
- sdk_sled_gps: Done.
- sdk_sled_lte: Done.
- sdk_sled_lte_stress: Done.
- sdk_swr: Done.
- sdk_wdt: Done.
- config_tool: Done.
"""
from time import sleep

import pytest

from lannerpsp import *

DELAY_TIME = 2  # Seconds.


class TestPSP:

    def test_init(self):
        with PSP():
            pass


class TestDLL:
    dll = DLL()

    def test_get_version(self):
        version = self.dll.get_version()
        assert version.dll_major == 2
        assert version.dll_minor == 1
        assert version.dll_build == 2
        assert version.platform_id == "LEB-7242"
        assert version.board_major == 1
        assert version.board_minor == 0
        assert version.board_build == 2

    def test_get_bios_id(self):
        assert self.dll.get_bios_id() == 'LEB-7242B BIOS V1.12 "03/09/2022"'


class TestHWM:
    hwm = HWM()

    def test_list_supported_sensors(self):
        supported_sensors = self.hwm.list_supported_sensors()
        assert sorted([s.sid for s in supported_sensors]) == sorted([0, 2, 4, 7, 8, 11, 12])
        assert supported_sensors[0].sid == 0
        assert supported_sensors[0].name == "HWMID_TEMP_CPU1"
        assert supported_sensors[1].sid == 2
        assert supported_sensors[1].name == "HWMID_TEMP_SYS1"
        assert supported_sensors[2].sid == 4
        assert supported_sensors[2].name == "HWMID_VCORE_CPU1"
        assert supported_sensors[3].sid == 7
        assert supported_sensors[3].name == "HWMID_VOLT_P5V"
        assert supported_sensors[4].sid == 8
        assert supported_sensors[4].name == "HWMID_VOLT_P3V3"
        assert supported_sensors[5].sid == 11
        assert supported_sensors[5].name == "HWMID_VOLT_VBAT"
        assert supported_sensors[6].sid == 12
        assert supported_sensors[6].name == "HWMID_VOLT_DDRCH1"


class TestRFM:
    rfm = RFM()

    def test_get_sim_status(self):
        assert self.rfm.get_sim_status() in range(4)

    def test_set_sim_status(self):
        self.rfm.set_sim_status(3)
        assert self.rfm.get_sim_status() == 3
        sleep(DELAY_TIME)
        self.rfm.set_sim_status(2)
        assert self.rfm.get_sim_status() == 2
        sleep(DELAY_TIME)
        self.rfm.set_sim_status(1)
        assert self.rfm.get_sim_status() == 1
        sleep(DELAY_TIME)
        self.rfm.set_sim_status(0)
        assert self.rfm.get_sim_status() == 0
        sleep(DELAY_TIME)

    def test_get_power_status(self):
        assert self.rfm.get_power_status() in range(4)

    def test_set_power_status(self):
        self.rfm.set_power_status(0)
        assert self.rfm.get_power_status() == 0
        sleep(DELAY_TIME)
        self.rfm.set_power_status(1)
        assert self.rfm.get_power_status() == 1
        sleep(DELAY_TIME)
        self.rfm.set_power_status(2)
        assert self.rfm.get_power_status() == 2
        sleep(DELAY_TIME)
        self.rfm.set_power_status(3)
        assert self.rfm.get_power_status() == 3
        sleep(DELAY_TIME)

    def test_set_sim_status_out_of_range(self):
        with pytest.raises(PSPInvalid):
            self.rfm.set_sim_status(-1)
        with pytest.raises(PSPError):
            self.rfm.set_sim_status(666)

    def test_set_power_status_out_of_range(self):
        with pytest.raises(PSPError):
            self.rfm.set_power_status(-1)
        with pytest.raises(PSPError):
            self.rfm.set_power_status(666)


class TestSystemLED:
    system_led = SystemLED()

    def test_green(self):
        with pytest.raises(PSPError):
            self.system_led.green()
        sleep(DELAY_TIME)

    def test_red(self):
        self.system_led.red()
        sleep(DELAY_TIME)

    def test_off(self):
        self.system_led.off()
        sleep(DELAY_TIME)


class TestGPSStatusLED:
    gps_status_led = GPSStatusLED()

    def test_on(self):
        self.gps_status_led.on()
        sleep(DELAY_TIME)

    def test_blink(self):
        self.gps_status_led.blink()
        sleep(DELAY_TIME)

    def test_off(self):
        self.gps_status_led.off()
        sleep(DELAY_TIME)


class TestLTEStatusLED:
    lte_status_led = LTEStatusLED()

    def test_red(self):
        self.lte_status_led.red()
        sleep(DELAY_TIME)

    def test_red_blink(self):
        self.lte_status_led.red_blink()
        sleep(DELAY_TIME)

    def test_green(self):
        self.lte_status_led.green()
        sleep(DELAY_TIME)

    def test_green_blink(self):
        self.lte_status_led.green_blink()
        sleep(DELAY_TIME)

    def test_yellow(self):
        self.lte_status_led.yellow()
        sleep(DELAY_TIME)

    def test_yellow_blink(self):
        self.lte_status_led.yellow_blink()
        sleep(DELAY_TIME)

    def test_off(self):
        self.lte_status_led.off()
        sleep(DELAY_TIME)


class TestLTEStressLED:
    lte_stress_led = LTEStressLED()

    def test_set_strength(self):
        self.lte_stress_led.set_strength(90)
        sleep(DELAY_TIME)
        self.lte_stress_led.set_strength(78)
        sleep(DELAY_TIME)
        self.lte_stress_led.set_strength(66)
        sleep(DELAY_TIME)
        self.lte_stress_led.set_strength(54)
        sleep(DELAY_TIME)
        self.lte_stress_led.set_strength(42)
        sleep(DELAY_TIME)
        self.lte_stress_led.set_strength(30)
        sleep(DELAY_TIME)
        self.lte_stress_led.set_strength(18)
        sleep(DELAY_TIME)
        self.lte_stress_led.set_strength(6)
        sleep(DELAY_TIME)

    def test_set_strength_out_of_range(self):
        with pytest.raises(PSPInvalid):
            self.lte_stress_led.set_strength(-1)
        with pytest.raises(PSPInvalid):
            self.lte_stress_led.set_strength(101)

    def test_off(self):
        self.lte_stress_led.off()
        sleep(DELAY_TIME)


class TestWDT:
    wdt = WDT()

    def test_get_info(self):
        wdt_info = self.wdt.get_info()
        assert wdt_info.type == "SuperIO"
        assert wdt_info.max_count == 255
        assert wdt_info.is_minute_support is True

    def test_enable(self):
        self.wdt.enable(10, 2)

    def test_reset(self):
        # PSP version 2.1 has a bug in the WDT `reset` function
        # that stops the timer like `disable`.
        self.wdt.reset()

    def test_disable(self):
        self.wdt.disable()

    def test_config_out_of_range(self):
        with pytest.raises(PSPInvalid):
            self.wdt.config(256)
        with pytest.raises(PSPInvalid):
            self.wdt.config(10, 3)

    def test_integration(self):
        with pytest.raises(PSPError):
            self.wdt.reset()
        self.wdt.enable(200)
        sleep(DELAY_TIME)
        self.wdt.reset()
        sleep(DELAY_TIME)
        self.wdt.disable()


class TestComPort:

    def test_set_all(self):
        com1 = COMPort(1)
        com1.set_mode(232)
        com1.set_mode(422)
        com1.set_mode(485)
        com1.set_termination(True)
        com1.set_termination(False)
        with pytest.raises(PSPNotSupport):
            com1.get_info()

    def test_init_out_of_range(self):
        with pytest.raises(PSPInvalid) as e:
            COMPort(2)
        assert str(e.value) == "'num' can only be set to (1) on this platform"

    def test_set_mode_out_of_range(self):
        com1 = COMPort(1)
        with pytest.raises(ValueError):
            com1.set_mode(666)



import logging
from ctypes import byref
from time import sleep
from typing import Any, Dict, NamedTuple

from .core import PSP, get_psp_exc_msg
from .exc import (
    PSPError,
    PSPNotSupport,
)
from .lmbinc import (
    AxisRawData,
    ERR_NotSupport,
    ERR_Success,
)
from .sdk_dll import DLL

logger = logging.getLogger(__name__)

SUPPORTED_PLATFORMS = ("V3S", "V6S",)
UNSUPPORTED_PLATFORMS = ("LEB-7242", "LEC-7230", "NCA-2510",)


class GSRDataModel(NamedTuple):
    """To store G-Sensor data."""
    g_range: int
    raw_x: int
    raw_y: int
    raw_z: int
    mg_x: float
    mg_y: float
    mg_z: float

    def to_dict(self) -> Dict[str, Any]:
        """Convert to dict."""
        return dict(self._asdict())


class GSROffsetModel(NamedTuple):
    """To store G-Sensor offset data."""
    raw_x: int
    raw_y: int
    raw_z: int

    def to_dict(self) -> Dict[str, Any]:
        """Convert to dict."""
        return dict(self._asdict())


class GSR:
    """
    G-Sensor.

    :param bool check_platform:
        Set to :data:`True` to check if the platform supports this feature.
        Defaults to :data:`False` for better compatibility.
    """

    def __init__(self, check_platform: bool = False) -> None:
        self._version = DLL().get_version()
        if not check_platform:
            return
        if self._version.platform_id in SUPPORTED_PLATFORMS:
            pass
        elif self._version.platform_id in UNSUPPORTED_PLATFORMS:
            raise PSPNotSupport("Not supported on this platform")
        else:
            raise NotImplementedError

    def get_data(self) -> GSRDataModel:
        """
        Get the X-Axis, Y-Axis and Z-Axis data.

        Example:

        .. code-block:: pycon

            >>> gsr = GSR()
            >>> gsr_data = gsr.get_data()
            >>> gsr_data.g_range
            2
            >>> gsr_data.raw_x
            -4
            >>> gsr_data.raw_y
            -9
            >>> gsr_data.raw_z
            -214
            >>> gsr_data.mg_x
            -0.03137254901960784
            >>> gsr_data.mg_y
            -0.07058823529411765
            >>> gsr_data.mg_z
            -1.6784313725490196
            >>> gsr_data.to_dict()
            {'g_range': 2, 'raw_x': -3, 'raw_y': -9, 'raw_z': -218, 'mg_x': -0.023529411764705882, 'mg_y': -0.07058823529411765, 'mg_z': -1.7098039215686274}

        :return: X/Y/Z Axis data and range with ±g
        :rtype: GSRDataModel
        :raises PSPNotSupport: This platform does not support this function.
        :raises PSPError: General PSP functional error.
        """
        stu_raw_data = AxisRawData()
        with PSP() as psp:
            i_ret = psp.lib.LMB_GSR_GetAxisData(byref(stu_raw_data))
        msg = get_psp_exc_msg("LMB_GSR_GetAxisData", i_ret)
        if i_ret == ERR_Success:
            pass
        elif i_ret == ERR_NotSupport:
            raise PSPNotSupport(msg)
        else:
            raise PSPError(msg)

        if stu_raw_data.w_g_range == 2:
            f_mg_step = 2 / 255
        elif stu_raw_data.w_g_range == 4:
            f_mg_step = 4 / 255
        elif stu_raw_data.w_g_range == 8:
            f_mg_step = 8 / 255
        elif stu_raw_data.w_g_range == 16:
            f_mg_step = 16 / 255
        else:
            raise PSPError(f"'w_g_range' = {stu_raw_data.w_g_range}")

        f_x_mg = stu_raw_data.w_x_axis * f_mg_step
        f_y_mg = stu_raw_data.w_y_axis * f_mg_step
        f_z_mg = stu_raw_data.w_z_axis * f_mg_step

        return GSRDataModel(g_range=stu_raw_data.w_g_range,
                            raw_x=stu_raw_data.w_x_axis,
                            raw_y=stu_raw_data.w_y_axis,
                            raw_z=stu_raw_data.w_z_axis,
                            mg_x=f_x_mg,
                            mg_y=f_y_mg,
                            mg_z=f_z_mg)

    def get_offset(self) -> GSROffsetModel:
        """
        Get the X-Axis, Y-Axis and Z-Axis offset data.

        Example:

        .. code-block:: pycon

            >>> gsr = GSR()
            >>> gsr_offset = gsr.get_offset()
            >>> gsr_offset.raw_x
            0
            >>> gsr_offset.raw_y
            0
            >>> gsr_offset.raw_z
            0
            >>> gsr_offset.to_dict()
            {'raw_x': 0, 'raw_y': 0, 'raw_z': 0}

        :return: X/Y/Z Axis offset data
        :rtype: GSROffsetModel
        :raises PSPNotSupport: This platform does not support this function.
        :raises PSPError: General PSP functional error.
        """
        stu_raw_data = AxisRawData()
        with PSP() as psp:
            i_ret = psp.lib.LMB_GSR_GetAxisOffset(byref(stu_raw_data))
        msg = get_psp_exc_msg("LMB_GSR_GetAxisOffset", i_ret)
        if i_ret == ERR_Success:
            return GSROffsetModel(
                raw_x=stu_raw_data.w_x_axis,
                raw_y=stu_raw_data.w_y_axis,
                raw_z=stu_raw_data.w_z_axis,
            )
        elif i_ret == ERR_NotSupport:
            raise PSPNotSupport(msg)
        else:
            raise PSPError(msg)

    def test(self) -> None:
        """
        For testing.

        Example:

        .. code-block:: pycon

            >>> gsr = GSR()
            >>> gsr.test()
            ---------> 0
            stuRawData.wRange= ±2g
            Raw=-5  , X-Axis= -0.03921569
            Raw=-9  , Y-Axis= -0.07058824
            Raw=-216        , Z-Axis= -1.69411765
            Offset X-Axis=0
            Offset Y-Axis=0
            Offset Z-Axis=0
            ---------> 1
            stuRawData.wRange= ±2g
            Raw=-5  , X-Axis= -0.03921569
            Raw=-8  , Y-Axis= -0.06274510
            Raw=-215        , Z-Axis= -1.68627451
            Offset X-Axis=0
            Offset Y-Axis=0
            Offset Z-Axis=0
            ---------> 2
            stuRawData.wRange= ±2g
            Raw=-4  , X-Axis= -0.03137255
            Raw=-10 , Y-Axis= -0.07843137
            Raw=-216        , Z-Axis= -1.69411765
            Offset X-Axis=0
            Offset Y-Axis=0
            Offset Z-Axis=0
            .
            .
            .
            ---------> 99
            stuRawData.wRange= ±2g
            Raw=-7  , X-Axis= -0.05490196
            Raw=-9  , Y-Axis= -0.07058824
            Raw=-218        , Z-Axis= -1.70980392
            Offset X-Axis=0
            Offset Y-Axis=0
            Offset Z-Axis=0
        """
        stu_raw_data = AxisRawData()
        with PSP() as psp:
            for i in range(100):
                print(f"---------> {i:d}")

                # Get accel data.
                i_ret = psp.lib.LMB_GSR_GetAxisData(byref(stu_raw_data))
                if i_ret != ERR_Success:
                    msg = get_psp_exc_msg("LMB_GSR_GetAxisData", i_ret)
                    print(f"\033[1;31m{msg}\033[0m")
                else:
                    print(f"stuRawData.wRange= ±{stu_raw_data.w_g_range:d}g")

                    if stu_raw_data.w_g_range == 2:
                        f_mg_step = 2 / 255
                    elif stu_raw_data.w_g_range == 4:
                        f_mg_step = 4 / 255
                    elif stu_raw_data.w_g_range == 8:
                        f_mg_step = 8 / 255
                    elif stu_raw_data.w_g_range == 16:
                        f_mg_step = 16 / 255

                    f_x_mg = stu_raw_data.w_x_axis * f_mg_step
                    f_y_mg = stu_raw_data.w_y_axis * f_mg_step
                    f_z_mg = stu_raw_data.w_z_axis * f_mg_step

                    print(f"Raw={stu_raw_data.w_x_axis:d}\t, X-Axis= {f_x_mg:03.8f}")
                    print(f"Raw={stu_raw_data.w_y_axis:d}\t, Y-Axis= {f_y_mg:03.8f}")
                    print(f"Raw={stu_raw_data.w_z_axis:d}\t, Z-Axis= {f_z_mg:03.8f}")

                # Get offset.
                i_ret = psp.lib.LMB_GSR_GetAxisOffset(byref(stu_raw_data))
                if i_ret != ERR_Success:
                    msg = get_psp_exc_msg("LMB_GSR_GetAxisOffset", i_ret)
                    print(f"\033[1;31m{msg}\033[0m")
                else:
                    print(f"Offset X-Axis={stu_raw_data.w_x_axis:d}")
                    print(f"Offset Y-Axis={stu_raw_data.w_y_axis:d}")
                    print(f"Offset Z-Axis={stu_raw_data.w_z_axis:d}")

                sleep(0.5)



import logging
from ctypes import addressof, byref, c_char_p, c_int8, sizeof
from mmap import mmap, PROT_READ, MAP_SHARED
from typing import Any, Dict, NamedTuple

from .core import PSP, get_psp_exc_msg
from .exc import (
    PSPError,
    PSPNotOpened,
    PSPNotSupport,
)
from .lmbinc import (
    DLLVersion,
    ERR_Success,
    ERR_NotOpened,
    ERR_NotSupport,
)

logger = logging.getLogger(__name__)


class DLLVersionModel(NamedTuple):
    """To store DLL and Board Library version information."""
    dll_major: int
    dll_minor: int
    dll_build: int
    platform_id: str
    board_major: int
    board_minor: int
    board_build: int

    def __str__(self) -> str:
        return f"PSP/SDK version: {self.dll_major}.{self.dll_minor}.{self.dll_build}\n" \
               f"IODRV   version: {self.platform_id}.{self.board_major}.{self.board_minor}.{self.board_build}"

    def to_dict(self) -> Dict[str, Any]:
        """Convert to dict."""
        return dict(self._asdict())


class DLL:
    """
    Dynamic Link Library.
    """

    def __init__(self) -> None:
        pass

    def get_version(self) -> DLLVersionModel:
        """
        Load the Lanner board-level library.

        Example:

        .. code-block:: pycon

            >>> dll = DLL()
            >>> version = dll.get_version()
            >>> version
            DLLVersionModel(dll_major=2, dll_minor=1, dll_build=2, platform_id='LEB-7242', board_major=1, board_minor=0, board_build=2)
            >>> version.to_dict()
            {'dll_major': 2, 'dll_minor': 1, 'dll_build': 2, 'platform_id': 'LEB-7242', 'board_major': 1, 'board_minor': 0, 'board_build': 2}
            >>> str(version)
            'PSP/SDK version: 2.1.2\\nIODRV   version: LEB-7242.1.0.2'
            >>> version.dll_major
            2
            >>> version.dll_minor
            1
            >>> version.dll_build
            2
            >>> version.platform_id
            'LEB-7242'
            >>> version.board_major
            1
            >>> version.board_minor
            0
            >>> version.board_build
            2

        :return: the DLL and Board Library version information
        :rtype: DLLVersionModel
        :raises PSPNotOpened: The library is not ready or opened yet.
        :raises PSPNotSupport: This platform does not support this function.
        :raises PSPError: General PSP functional error.
        """
        stu_dll_ver = DLLVersion()
        with PSP() as psp:
            i_ret = psp.lib.LMB_DLL_Version(byref(stu_dll_ver))
        msg = get_psp_exc_msg("LMB_DLL_Version", i_ret)
        if i_ret == ERR_Success:
            return DLLVersionModel(
                dll_major=stu_dll_ver.uw_dll_major,
                dll_minor=stu_dll_ver.uw_dll_minor,
                dll_build=stu_dll_ver.uw_dll_build,
                # https://stackoverflow.com/a/29293102/9611854
                platform_id=c_char_p(addressof(stu_dll_ver.str_platform_id)).value.decode(),
                board_major=stu_dll_ver.uw_board_major,
                board_minor=stu_dll_ver.uw_board_minor,
                board_build=stu_dll_ver.uw_board_build,
            )
        elif i_ret == ERR_NotOpened:
            raise PSPNotOpened(msg)
        elif i_ret == ERR_NotSupport:
            raise PSPNotSupport(msg)
        else:
            raise PSPError(msg)

    def get_bios_id(self) -> str:
        """
        Get the Lanner mother-board BIOS infromation.

        Example:

        .. code-block:: pycon

            >>> dll = DLL()
            >>> dll.get_bios_id()
            'LEB-7242B BIOS V1.12 "03/09/2022"'

        :return: the mother board BIOS information
        :rtype: str
        :raises PSPNotOpened: The library is not ready or opened yet.
        :raises PSPNotSupport: This platform does not support this function.
        :raises PSPError: General PSP functional error.
        """
        version = self.get_version()
        if version.dll_major == 2 and version.dll_minor in (0, 1, 2):
            # `sudo usermod -g kmem yourID`
            # `sudo busybox devmem 0x00ff58b 8 | xxd -r -p`
            key = "*LIID "
            with open("/dev/mem", "rb") as f:
                mem = mmap(f.fileno(), 0x10000, MAP_SHARED, PROT_READ, offset=0x000f0000)
            if mem is None:
                return ""
            if not mem.read(33 + len(key)).startswith(key.encode()):
                # not found "*LIID"
                # add here for BIOS uses traditional position F000:F58B
                mem.seek(0xF58B)
            return mem.read(33 + len(key)).decode().replace(key, "").strip().rsplit('"', 1)[0] + '"'
        else:
            str_bios_id = (c_int8 * 50)(*range(50))  # str_bios_id = create_string_buffer(50)
            with PSP() as psp:
                i_ret = psp.lib.LMB_DLL_BIOSID(str_bios_id, sizeof(str_bios_id))
            msg = get_psp_exc_msg("LMB_DLL_BIOSID", i_ret)
            if i_ret == ERR_Success:
                return c_char_p(addressof(str_bios_id)).value.decode().strip()
            elif i_ret == ERR_NotOpened:
                raise PSPNotOpened(msg)
            elif i_ret == PSPNotSupport:
                raise PSPNotSupport(msg)
            else:
                raise PSPError(msg)
